#!/usr/bin/perl -w

########################################################################################################################
#
# GetLive - perl script to get mail from hotmail (live) mailboxes.
#
# $Id: GetLive.pl,v 1.23 2007/06/24 17:37:41 jdla Exp $
# $Name: RELEASE_0_8 $
#
# Copyright (C) 2007 Jos De Laender <jos.de_laender@pandora.be>
#
# This work is inspired and partly reuses code from 
# gotmail :
#   Copyright (C) 2000-2003 Peter Hawkins <peterhawkins@ozemail.com.au>
#   Copyright (C) 2005 Jon Phillips <jon@rejon.org>
#   Copyright (C) 2005 Michael Ziegler.
#   Copyright (C) 2005-2006 Jos De Laender <jos.de_laender@pandora.be>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
########################################################################################################################

use strict;
use File::Spec;
use URI::Escape;

########################################################################################################################
#
# Global constants and variables.
#
########################################################################################################################

my $ProgramName = "GetLive";
my $Revision    = '$Revision: 1.23 $';		# Meant for RCS.

# Constants of configuration.
my $Proxy                   = "";
my $ProxyAuth               = "";
my $Login                   = "";
my $Password                = "";
my $Mode                    = 'Live';               # The live hotmail boxes : 'Live'. Otherwise 'Dead'
my $Domain                  = 'hotmail.com';
my $CurlCommand             = 'curl -k';
my $Verbosity               = 1; 									  # 0:Silent; 1:Normal; 2:Verbose; 10:debug; 100:heavy debug 
my $MailProcessor           = '/usr/bin/procmail';	# Any program taking mbox formatted at stdin will do.
my $DownloadedIdsFile       = "";                   # Local file with Ids of already downloaded messages.
my $RetryLimit              = 2;
my $MarkRead                = "No";                 # No,Yes : But never when downloaded before !
my $FetchOnlyUnread         = "No";                 # If Yes, only messages marked unread are downloaded.
                                                    # Unlike gotmail, this is completely orthogonal to the
                                                    # DownloadedIdsFile, i.e. it is the one or the other.
my $MoveToFolder            = "";                   # The name of the folder to move to after the download. "" is not.
                                                    # If it begins with @ it is reference to a filename that
                                                    # contains the folder to move to. This is a hook for 
                                                    # autoclassifying the mail on the server, including spam filtering.

# Files in a temporary directory.
my $TmpDir             = File::Spec->tmpdir() . "/$ProgramName.$$";
my $TmpCurlHeadersFile = "$TmpDir/Headers";
my $TmpCookiesFile     = "$TmpDir/Cookies";
my $TmpFormDataFile    = "$TmpDir/Form";
my $TmpCurlStderrFile  = "$TmpDir/CurlStderr";
my $TmpCurlStdoutFile  = "$TmpDir/CurlStdout";
my $TmpCurlTraceFile   = "$TmpDir/CurlTrace";

# Messages retrieved from a folder.
my $NrMessagesDetected = 0;
my $NrEditsDetected    = 0;    # Messages in edit mode (typically concept folder but can be elsewhere too.
my @MessagesFrom    = ();
my @MessagesSubject = ();
my @MessagesId      = ();
my @MessagesHref    = ();
my @MessagesRead    = ();

# Various variables.
my $BaseUrl; 						# The one in the logged in screen used for fetching folders.

my @FolderHrefs      = ();	# The Hrefs found for the different folders.
my @FolderNames      = ();	# The names found for the different folders.
my @FolderNrMessages = ();	# The number of messages found for the different folders.
my $NrFolders        = 0;		# The number of folders found.

my %FoldersToProcess = ();  # The folders to process (empty will be considered as all). Otherwise FolderName=>1 assoc.

my $CurlRun    = 0;			# Increased with each Curl run. Basically for debug reasons.
my $ConfigFile;

########################################################################################################################
# 
# Catchall signal handler. Just observes death and cleans up the mess.
#
########################################################################################################################

$SIG{INT} = $SIG{TERM} = $SIG{__DIE__} = sub {
  my($Text) = @_;
	print STDERR "$ProgramName died with message: '$Text'.";
	CleanTempFiles();
	exit(1);
};

########################################################################################################################
# 
# Display some text.
# First parameter : text to be displayed.
# Then a number of named parameters that are optional. 
# See %args.
#
########################################################################################################################

sub Display($%) {
	my $Text = shift;
	my %Args = (MinVerbosity					=> 0,
							stderr								=> 0,
							@_);

  # stderr messages are under no circumstances suppressed.
	if ($Args{'stderr'}) {
		print STDERR $Text;
		return;
	}

	# Filter out the ones for which the verbosity is too high.
	return if ($Args{'MinVerbosity'} > $Verbosity);

	# And finally print ;-)
  # Stdout is flushed immediate , not to miss error messages.
  my $WasSelected = select(STDOUT);
  $|=1;
  select($WasSelected);

	print STDOUT $Text;

	return;
}

########################################################################################################################
# 
# Display the introduction text.
# Text as argument, stderr as optional named argument to redirect to stderr.
#
########################################################################################################################

sub DisplayIntroText(%) {
	my %Args = (stderr => 0,
              MinVerbosity => 1,
              @_);
	my $Text = 
	  "\n".
	  "$ProgramName $Revision Copyright (C)2007 Jos De Laender.\n".
		"$ProgramName comes with ABSOLUTELY NO WARRANTY.\n".
		"This is free software, and you are welcome to redistribute it\n".
		"under certain conditions; see the file License for details.\n".
		'$Name: RELEASE_0_8 $' . "\n".
		'$Id: GetLive.pl,v 1.23 2007/06/24 17:37:41 jdla Exp $' . "\n".
    "Running at ".localtime(time)." for user $Login.\n";
	Display($Text,%Args);
}

########################################################################################################################
# 
# This is only called in error conditions. Output will go to stderr.
#
########################################################################################################################

sub DisplayUsageAndExit() {
	DisplayIntroText(stderr => 1);
	Display("Usage: $ProgramName --config-file ConfigFile [--verbosity -1..100]\n",stderr => 1);
	exit(1);
}

########################################################################################################################
# 
# Parse the command line
#
########################################################################################################################

sub ParseArgs() {
  my $ArgvAsString =  join(" ",@ARGV);

  # --config-file is a mandatory argument.
  if ($ArgvAsString !~ m/--config-file\s+([\w\/\\~\.]+)/si) {
    DisplayUsageAndExit();
  }
  $ConfigFile =  $1;
  $ArgvAsString = $` . $';	  # The matched stuff removed.

  # --verbosity is an optional argument.
  if ($ArgvAsString =~ m/--verbosity\s+(\d+)/si) {
    $Verbosity = $1;
    $ArgvAsString = $` . $';	# The matched stuff removed.
  }
  # Should have no other arguments.
  $ArgvAsString =~ s/\s//sg;
  if ($ArgvAsString ne "") {
    Display("Wrong command line arguments '$ArgvAsString'.\n",stderr => 1);
    DisplayUsageAndExit();
  }
}

########################################################################################################################
# 
# Parse the Configuration File
#
########################################################################################################################

sub ParseConfig {

	open (CONFIG,$ConfigFile) || die "Configuration file '$ConfigFile' could not be opened : $!.";

	# Parse the file
	while (<CONFIG>) {
		my $Line = $_;
		next if ($Line =~ /^#/); # Comment.
		next if ($Line =~ /^\s*$/); # Empty line.
		if (not $Line =~ m/^([a-zA-Z0-9-_]+)/) {
			Display("Wrong configuration line : '$_'.\n",stderr=>1);
			DisplayUsageAndExit();
		}
		my $Option      = $1;
		my $OptionValue = "";
		$Line           = $'; # The remaining of the line.
		if (not $Line =~ m/\s*=\s*\S+/) {
		  Display("Wrong configuration line : '$_' (no value).\n",stderr => 1);
			DisplayUsageAndExit();
		}
		# Remove equals sign and leading, trailing whitespace.
		$Line =~ s/=//;		
		$Line =~ s/^\s+|\s+$//g;
		$OptionValue = $Line;

    if ($Option =~ m/^UserName$/i) {
      $Login = $OptionValue;
    } elsif ($Option =~ m/^Password$/i) {
      $Password = $OptionValue;
    } elsif ($Option =~ m/^Mode$/i) {
      $Mode = $OptionValue;
      # For backward compatibility.
      if ($Mode =~ m/Classic/i) {
        warn "\nMode = Classic is depreciated, use Mode = Dead in the config file !\n";
        $Mode = "Dead";
      }
    } elsif ($Option =~ m/^Domain$/i) {
      $Domain = $OptionValue;
    } elsif ($Option =~ m/^Proxy$/i) {
      $Proxy = $OptionValue;
    } elsif ($Option =~ m/^ProxyAuth$/i) {
      $ProxyAuth = $OptionValue;
    } elsif ($Option =~ m/^Downloaded$/i) {
      $DownloadedIdsFile = $OptionValue;
    } elsif ($Option =~ m/^RetryLimit$/i) {
      $RetryLimit = $OptionValue;
    } elsif ($Option =~ m/^Processor$/i) {
      $MailProcessor = $OptionValue;
    } elsif ($Option =~ m/^CurlBin$/i) {
      $CurlCommand = $OptionValue;
    } elsif ($Option =~ m/^Folder$/i) {
      $FoldersToProcess{lc $OptionValue} = 1;
    } elsif ($Option =~ m/^FetchOnlyUnread$/i) {
      $FetchOnlyUnread = $OptionValue;
    } elsif ($Option =~ m/^MarkRead$/i) {
      $MarkRead = $OptionValue;
    } elsif ($Option =~ m/^MoveToFolder$/i) {
      $MoveToFolder = $OptionValue;
    } else {
		  Display("Wrong configuration line : '$_' (unknown option).\n",stderr=>1);
			DisplayUsageAndExit();
    }
	}
	close(CONFIG);

  # Some sanitychecks.
  if ($Login eq "") {
    Display("UserName should be specified in the configuration file.\n",stderr=>1); 
    DisplayUsageAndExit(); 
  }
  if ($Password eq "") {
    Display("Password should be specified in the configuration file.\n",stderr=>1); 
    DisplayUsageAndExit(); 
  }
  if ($FetchOnlyUnread !~ m/^(No|Yes)$/i) {
    Display("FetchOnlyUnread should take No or Yes as argument in the configuration file.\n",stderr=>1);
    DisplayUsageAndExit(); 
  }
  if ( ($FetchOnlyUnread =~ m/^No$/i) && ($DownloadedIdsFile eq "") ) {
    Display("Downloaded should be specified in the configuration file.\n",stderr=>1); 
    DisplayUsageAndExit(); 
  } 
  if ( ($FetchOnlyUnread =~ m/^Yes$/i) && ($DownloadedIdsFile ne "") ) {
    Display("Downloaded should not be specified in the configuration file when FetchOnlyUnread=Yes.\n",stderr=>1); 
    DisplayUsageAndExit(); 
  } 
  if ($MarkRead !~ m/^(No|Yes)$/i) {
    Display("MarkRead should take No or Yes as argument in the configuration file.\n",stderr=>1);
    DisplayUsageAndExit(); 
  }
}

########################################################################################################################
# 
# Clean up any temporary files which are collected in a temporary directory.
#
########################################################################################################################

sub CleanTempFiles() {
  return if ($Verbosity >9);		# Considered debug mode and thus keep the files !
  return if (! -e $TmpDir);     # We're even not at the point that the tmpdir exists ...
  opendir (TMPDIR,$TmpDir) || die "Could not open '$TmpDir' : $!.";
  while (my $FileName = readdir(TMPDIR)) {
    next if $FileName =~ m/^\.$/;			# Not the .
    next if $FileName =~ m/^\.\.$/;		# Nor .. directory
    unlink("$TmpDir/$FileName") || warn "Could not unlink $TmpDir.$FileName : $!";
  }
  closedir (TMPDIR);
  # Finally get rid of the temporary directory itself.
  rmdir($TmpDir) || warn "Could not unlink $TmpDir";
}

########################################################################################################################
# 
# Unescape html characters, widechars become blank along the conversion.
#
# Based on a function with copyright: Bryant H. McGill - 11c Lower Dorset Street, Dublin 1, Ireland
# Use Terms: Free for non-commercial use, commercial use with notification.
#
########################################################################################################################

sub HtmlUnescape($) {
  my $String = shift;
  $String =~ s[&(.*?);]{
    local $_ = $1;
    /^amp$/i ? "&" :
    /^quot$/i ? '"' :
    /^gt$/i ? ">" :
    /^lt$/i ? "<" :
    /^nbsp$/i ? " " :
    /^#(\d+)$/ ? ($1>255 ? "":chr($1)) :
    /^#x([0-9a-f]+)$/i ? (hex($1)>255 ? "": chr(hex($1))) :
    $_
    }gex;
  return $String;
}

########################################################################################################################
# 
# Get a html page, basically via curl. 
# Returns the page as one big string.
# Returns a second string with the latest url.
# The parameters should be reasonably clear. FollowForward will follow a redirection.
#
########################################################################################################################

sub GetPage($%) {
  my %Args = (Url           => "",
							CurlDataArg		=> "",
							FollowForward	=> 0,
							@_);
  my $Url           = $Args{'Url'};
  my $CurlDataArg   = $Args{'CurlDataArg'};
  my $FollowForward = $Args{'FollowForward'};

	$CurlRun++;

  my $OptionsToCurl = "";

	if ($Proxy) {
    $OptionsToCurl .= "--proxy $Proxy "; 
  }
	if ($ProxyAuth) { 
    $OptionsToCurl .= "--proxy-user $ProxyAuth "; 
  }

  # The files with the Cookies.
  $OptionsToCurl .= "-b $TmpCookiesFile -c $TmpCookiesFile ";

	if ($CurlDataArg ne "") { 
    $OptionsToCurl .= "--data \"$CurlDataArg\" ";
  }

	# Curl is put silent (but with error output) 
	# when not interactive or low verbosity.
	if ( (not -t STDOUT) || ($Verbosity <= 1) ) { 
    $OptionsToCurl .= "-s -S " 
  }

	if ($Verbosity > 9) { 
    $OptionsToCurl .= "-v --trace $TmpCurlTraceFile.$CurlRun" 
  }

	# JDLA curl outputs info via stderr. Catched in file and appended
	# to stdout output in debug mode.
  my $CommandLine = 
		"$CurlCommand --stderr $TmpCurlStderrFile.$CurlRun \'$Url\' " .
    "$OptionsToCurl -i -m 600 -D $TmpCurlHeadersFile.$CurlRun " .
		"-A \"Mozilla/4.73 [en] (Win98; I)\"";
  Display("Curl run $CurlRun.\nCommandLine : '$CommandLine'.\n", MinVerbosity => 10);
  my $NrTries = 0;
	my @CurlOutput = ();
	while (!@CurlOutput && $NrTries++ < $RetryLimit) {
		Display("Trying [$NrTries/$RetryLimit].\n",MinVerbosity => 2);
		@CurlOutput = `$CommandLine`;
		# Copy output. Only in very high debug levels.  # We have it in file anyway.
    if ($Verbosity > 99) {	# The if around makes it a bit more efficient over the loop.
		  foreach my $Line (@CurlOutput) { 
		    Display($Line,MinVerbosity => 100); 
      }
    }
		open (CURL_STDERR,"$TmpCurlStderrFile.$CurlRun") || die "Could not open $TmpCurlStderrFile.$CurlRun : $!.";
		# Copy curl stderr.
		Display("\nstderr of curl :\n",MinVerbosity => 10);
		while(<CURL_STDERR>) {
			my $Line = $_;
			my $PasswordToBlank = uri_escape($Password,"^A-Za-z");
			$Line =~ s/$PasswordToBlank/YouThinkThisIsThePassword/g;
		  Display("$Line",MinVerbosity => 10);
		}
		close(CURL_STDERR);
		Display("\nEnd of stderr of curl.\n",MinVerbosity => 10);
	}

  # In debug mode (Verbosity>9) we copy the output to a file.
  if ($Verbosity > 9) {
    open (CURL_STDOUT,">$TmpCurlStdoutFile.$CurlRun") || 
		  die "Could not open $TmpCurlStdoutFile.$CurlRun : $!.";
    print CURL_STDOUT @CurlOutput;
    close(CURL_STDOUT);
  }

	if (!@CurlOutput && $NrTries > $RetryLimit) {
    Display("Curl run $CurlRun.\nCommandLine : '$CommandLine'.\n",stderr => 1);
		die("An error was encountered getting the page.");
	}

	# Redirect search in headers.
	my $Redirection = "";
  open (CURL_HEADERS,"$TmpCurlHeadersFile.$CurlRun") || die "Could not open $TmpCurlHeadersFile.$CurlRun : $!.";
	while (<CURL_HEADERS>) {
		if (m/^Location: (\S+)\s/) {
			$Redirection = $1;
      last;
		}
	}
  close(CURL_HEADERS);

	# If we have been asked to follow Location: headers
	if ($FollowForward) {
		if ($Redirection ne "") {
      if ($Redirection !~ m/^http.*/i) {
        if ($Url =~ m/(http?:\/\/[^\/]+)\//i) {
          $Redirection = $1 . $Redirection;
        }
      }
			Display("Following redirect to $Redirection.\n",MinVerbosity => 2);
			return &GetPage(Url => $Redirection,FollowForward => $FollowForward);
		}
	}

	return (join("",@CurlOutput),$Url);
}

########################################################################################################################
# 
# Do the HotMail login process - log in until we have the URL of the inbox.
#
########################################################################################################################

sub Login() {

	Display("Getting hotmail index loginpage.\n", MinVerbosity =>2);

  my ($LoginPageAsString,$GetPageUrl) = GetPage(Url => "http://www.hotmail.com/",FollowForward => 1);
	if ($LoginPageAsString !~ m/<form.*f1.*action=\"(\S+)\".*>/i) {
		die "Page doesn't contain an action in the expected place.";
	}
  my $LoginUrl = $1;

  ## The actual action of the form is replace by javascript
  ## (authentication depends on the domain of your user id).
  ## and arguments are appended in javascript
	if( $LoginPageAsString =~ m/g_DO\["\Q$Domain\E"\]\s*=\s*"([^"]+)\"/ ) {
		$LoginUrl = $1;
		Display("LoginUrl detected as '$LoginUrl'.\n", MinVerbosity => 10 );
	}
	if( $LoginPageAsString =~ m/g_QS\s*=\s*"([^"]+)\"/ ) {
	  $LoginUrl .= index($LoginUrl,'?') >= 0 ? '&' : '?';
		$LoginUrl .= $1;
		Display("LoginUrl extended to  '$LoginUrl'.\n", MinVerbosity => 10 );
		}

  ## Get (or calculate) the hidden fields of the form 
	my $PPFT;
  if( $LoginPageAsString =~ m/<\s*input\s+.*name=\"PPFT\"(\s+id="\S+")?\s+value=\"(\S*)\"/ ) {
	  $PPFT = $2;
		Display("PPFT detected : '$PPFT'.\n",MinVerbosity => 10 );
	}
  my $PPSX;
	if( $LoginPageAsString =~ m/<\s*input\s+.*name=\"PPSX\"(\s+id="\S+")?\s+value=\"(\S*)\"/ ) {
	  $PPSX = $2;
		Display("PPSX detected : '$PPSX'.\n",MinVerbosity => 10 );
	}

	my $Padding = "BovenGentRijstEenzaamEnGrijsHetOudBelfort";
	my $PwdPad = substr( $Padding, 0, length($Padding)-length($Password) ); 
	Display("PwdPad constructed : '$PwdPad'.\n",MinVerbosity => 10 );

  # Construct the form with above in a temporary file.
  open (FORMFILE,">$TmpFormDataFile") || die "Could not open $TmpFormDataFile : $!.";
	print FORMFILE "PPSX=$PPSX" .
								 "\&PwdPad=$PwdPad" .
								 "\&login="  . uri_escape($Login . '@' . $Domain, "^A-Za-z") .
								 "\&passwd=" . uri_escape($Password, "^A-Za-z") .
								 "\&SI="     . uri_escape( '    Sign In    ' ) . 
								 "\&LoginOptions=3" .
								 "\&PPFT="   . uri_escape( $PPFT );
	close FORMFILE;
  
  # Second step of login. The form is provided as a curl --data argumetn.
	Display("Logging in.\n",MinVerbosity => 1);

	($LoginPageAsString,$GetPageUrl) = GetPage(Url => $LoginUrl,CurlDataArg => "\@$TmpFormDataFile",FollowForward => 1);
	if ($LoginPageAsString =~ /password is incorrect/i) {
		die("There was an error logging in. Please check that your username and password are correct.");
	}

	if ($LoginPageAsString !~ m/window\.location\.replace\(\"(.*)\"\);/i) { 
		die("Hotmail's login structure has changed! (redirloc).");
  }
	$LoginUrl = $1; 

  Display("LoginUrl 2 : '$LoginUrl'.\n",MinVerbosity => 10);
  # Following the redirect : Third step of login.
	Display("Following redirect.\n",MinVerbosity => 2);
  ($LoginPageAsString,$GetPageUrl) = GetPage(Url => $LoginUrl,FollowForward => 1);

  my $StillRedirectToGo = 1;

  if ($Mode =~ m/Dead/i) {
	  if ($GetPageUrl !~ m/(.+)\/dasp\/ua_info.asp\?pg=browser_limit[^&]*(&.+)/) {
      die "Could not find expected url. Might you have specified a 'Dead' mailbox while it is really a 'Live' one ?";
      }
		$LoginUrl = "$1\/HoTMail";
  } elsif ($Mode =~ m/Live/i) {
    if ($LoginPageAsString =~ m/<div\s+class=\"MeSection\"\s*>/i) {
      $StillRedirectToGo = 0;
      $LoginUrl = $GetPageUrl;
    } else {
      if ($LoginPageAsString !~ m/window\.location\.replace\(\"(.*)\"\);/i) { 
        die "Could not find expected url. Might you have specified a 'Live' mailbox while it is really a 'Dead' one ?";
      }
	    $LoginUrl = $1; 
    }
  } else {
    die "Currently unknown mode '$Mode'.";
  }

  if ($StillRedirectToGo) { 
    Display("LoginUrl 3 : '$LoginUrl'.\n",MinVerbosity => 10);
    # Following the redirect : Fourth step of login. This gives us finally the main page.
	  Display("Following redirect.\n",MinVerbosity => 2);
	  ($LoginPageAsString,$GetPageUrl) = GetPage(Url => $LoginUrl,FollowForward => 1);
  }
  
  if ($LoginUrl !~ m/(http[s]?:\/\/([^\/]+\/)+)/) {
    die "Could not detect BaseUrl.";
  } 
  $BaseUrl = $1;
  Display("BaseUrl : $BaseUrl.\n",MinVerbosity => 10);

	Display("Got MainPage.\n",MinVerbosity => 1);
}

########################################################################################################################
#
# Search for Cookie in the CookiesFile.
# Argument : The cookie to be found.
# Returns its value.
#
########################################################################################################################

sub FindCookie($) {
  my ($CookieToFind) = @_;
  open (COOKIES,$TmpCookiesFile) || die "Could not open '$TmpCookiesFile'.";
  while (<COOKIES>) {
    chomp;
    next if m/^#/;  # Comment
    next if m/^$/;  # Empty line.
    my @SplittedLine = split /\t/;
    if ($SplittedLine[5] eq $CookieToFind) {
      close COOKIES;
      return $SplittedLine[6];
    }
  }
  close COOKIES;
  return "";
}

########################################################################################################################
# 
# Move the email message to a folder.
# MessageIdx and FolderName as argument.
#
########################################################################################################################

sub MoveToFolder($$$) {
  my ($MessageIdx,$TargetFolderName,$SourceFolderIdx) = @_;
  my $MessageId = $MessagesId[$MessageIdx];
  my $MessageHref = $MessagesHref[$MessageIdx];

  # Find out which folder (the index in @FolderHrefs) is meant.
  my $TargetFolderIdx   = 0;
  my $TargetFolderFound = 0; 
  while ((not $TargetFolderFound) && $TargetFolderIdx<$NrFolders) {
    if (lc $TargetFolderName eq lc $FolderNames[$TargetFolderIdx]) {
      $TargetFolderFound = 1;
    } else {
      $TargetFolderIdx++;
    }
  }

  # Let's die the hard way if we do not find that folder.
  die "Folder with name '$TargetFolderName' used in MoveToFolder could not be located." unless $TargetFolderFound;
      
	Display("Moving email message to folder '$TargetFolderName'.\n",MinVerbosity => 1);

  my $Url;
  my $PostData = "";
  if ($Mode =~  m/Live/i) {
    # Detect the FolderId in the Href.
    if ($FolderHrefs[$TargetFolderIdx] !~ m/FolderId=([^&]*)&/i) {
      die "Could not detect FolderId in '$FolderHrefs[$TargetFolderIdx]'."
    }
    my $ToBox = $1;
    # Get the SourceHref
    if ($FolderHrefs[$SourceFolderIdx] !~ m/[^\?]+\?(.*)/) {
      die "Could not get SourceHref in '$FolderHrefs[$SourceFolderIdx]'.";
    }
    my $SourceHref = $1;
    my $MT = FindCookie("mt");
    $Url = "${BaseUrl}InboxLight.aspx?$SourceHref"; 
    $PostData = "__VIEWSTATE=&mt=$MT&query=&MoveMessageSelector=$ToBox".
                "&ToolbarActionItem=MoveMessageSelector&SelectedMessages=$MessageId";
  } elsif ($Mode =~ m/Dead/i) {
    # Detect the FolderId in the Href.
    if ($FolderHrefs[$TargetFolderIdx] !~ m/curmbox=([^&]*?)&/) {
      die "Could not detect FolderId in '$FolderHrefs[$TargetFolderIdx]'.";
    }
    my $ToBox = $1;
    # Get the SourceHref
    if ($FolderHrefs[$SourceFolderIdx] !~ m/([^\?]+)\?(.*)/) {
      die "Could not get SourceHref in '$FolderHrefs[$SourceFolderIdx]'.";
    }
    my $Exec  = $1;
    my $Param = $2;
    if ($Param !~ m/a=([^&]*)&/ ) {
      die "Could not get 'a' in '$Param'.";
    }
    my $A = $1;
    if ($Param !~ m/curmbox=([^&]*)&/ ) {
      die "Could not get 'curmbox' in '$Param'.";
    }
    my $CurmBox = $1;
    
    $Url = "${BaseUrl}$Exec\?a=$A";
    $PostData = "curmbox=$CurmBox&HrsTest=&a=$A&js=&_HMaction=MoveTo&wo=&tobox=$ToBox&ReportLevel=&rj=&DoEmpty=".
                "&SMMF=0&$MessageId=on";
  } else {
    die "Not supported mode '$Mode'.";
  }

  # Do The move ...
	my ($EmailPageAsString,$GetPageUrl) = GetPage(Url => $Url,CurlDataArg => $PostData); 
}

########################################################################################################################
# 
# Mark the email message as read
# MessageIdx as argument.
#
########################################################################################################################

sub MarkRead($) {
  my ($MessageIdx) = @_;
  my $MessageId = $MessagesId[$MessageIdx];
  my $MessageHref = $MessagesHref[$MessageIdx];

	Display("Marking email message as read.\n",MinVerbosity => 1);

  my $Url;
  if ($Mode =~  m/Live/i) {
    $Url = "${BaseUrl}ReadMessageLight.aspx?ReadMessageId=$MessageId";
  } elsif ($Mode =~ m/Dead/i) {
    $Url = "${BaseUrl}$MessageHref";
  } else {
    die "Not supported mode '$Mode'.";
  }
  # We just read the message as we would do via the web 
  # (without the raw=0 ! or comparable stuff to get a copy of the message in mbox format). 
  # This marks it of course as read.
	my ($EmailPageAsString,$GetPageUrl) = GetPage(Url => $Url); 
}

########################################################################################################################
# 
# Return the email message (mbox format) as one big string.
# MessageIdx and FolderName as argument.
#
########################################################################################################################

sub GetEmail($$) {
  my ($MessageIdx,$FolderName) = @_;
  my $MessageId = $MessagesId[$MessageIdx];
  my $MessageHref = $MessagesHref[$MessageIdx];

	Display("Getting email message.\n",MinVerbosity => 1);

  my $Url;
  if ($Mode =~  m/Live/i) {
    $Url = "${BaseUrl}GetMessageSource.aspx?msgid=$MessageId";
  } elsif ($Mode =~ m/Dead/i) {
    $Url = "${BaseUrl}$MessageHref\&raw=0";
  } else {
    die "Not supported mode '$Mode'.";
  }
	my ($EmailPageAsString,$GetPageUrl) = GetPage(Url => $Url);
	if ($EmailPageAsString !~ /<pre>[\s\n]*(.*?)<\/pre>/si) {
		die "Unable to download email message.";
	}
  $EmailPageAsString = $1;
  $EmailPageAsString =~ s/^[\s\n]*//;
  $EmailPageAsString = HtmlUnescape($EmailPageAsString); # Strips all HTML artifacts from the message body.
  $EmailPageAsString =~ s/\r\n/\n/gs; # Force unix line endings.

  # Fallback envelope sender and date, case it would not be in the message.
	my $FromAddress = "$Login\@$Domain";
	my $FromDate    = scalar gmtime;

	# Strip "From whoever" when found on the first line- the format is wrong for mbox files anyway.
	if ($EmailPageAsString =~ s/^From ([^ ]*) [^\n]*\n//s) { 
    $FromAddress = $1; 
  } elsif ($EmailPageAsString =~ m/^From:[^<]*<([^>]*)>/m) { 
    $FromAddress = $1;  
  }

	# Apply >From quoting
	$EmailPageAsString =~ s/^From ([^\n]*)\n/>From $1/gm;

	# If an mboxheader was desired, make up one
	if ($EmailPageAsString =~ m/^\t (\w+), (\d+) (\w+) (\d+) (\d+):(\d+):(\d+) ([+-]?.+)/m) {
		my $DayOfWeek = $1;
		my $Month     = $3;
		my $Day       = $2;
		my $Hour      = $5;
		my $Minute    = $6;
		my $Second    = $7;
		my $Year      = $4;
		my $TimeZone  = $8;

		# Put date in mboxheader in UTC time
		$Hour -= $TimeZone;
		while ($Hour < 0)  { $Hour += 24; }
		while ($Hour > 23) { $Hour -= 24; }

		$FromDate = sprintf ("%s %s %02d %02d:%02d:%02d %d",$DayOfWeek,$Month,$Day,$Hour,$Minute,$Second,$Year);
	}

	# Add an mbox-compatible header
  # And add some identifying headers.
	$EmailPageAsString =~ s/^/From $FromAddress $FromDate\nX-$ProgramName-Version: $Revision\nX-$ProgramName-Folder: $FolderName\nX-$ProgramName-User: $Login\n/;

	return $EmailPageAsString;
}

########################################################################################################################
# 
# Get the messages from the folder with Idx as argument.
# Switcher between Live and Dead variant.
# 
########################################################################################################################

sub GetMessagesFromFolder($) {
  my ($FolderIdx) = @_;
  if ($Mode =~ m/Live/i) {
    GetMessagesFromFolderLive($FolderIdx);
  } elsif ($Mode =~ m/Dead/i) {
    GetMessagesFromFolderDead($FolderIdx);
  } else {
    die "Not supported mode '$Mode'.";
  }
}

########################################################################################################################
# 
# Get the messages from the folder with Idx as argument.
# Live variant.
# 
########################################################################################################################

sub GetMessagesFromFolderLive($) {
  my ($FolderIdx)        = @_;
  my $FolderName         = $FolderNames[$FolderIdx];
  my $ReportedNrMessages = $FolderNrMessages[$FolderIdx];

	Display("Loading folder '$FolderName'.\n",MinVerbosity => 1);

	my $Page = 0;
	my $StillPageToGo = 1;

  # Reinitialize the global variable back to 0.
  $NrMessagesDetected = 0;
  $NrEditsDetected    = 0;
  

  # PageUrl starts with the information retrieved from the initial page.
  # In subsequent loops overwritten by NextPageHref.
  my $PageUrl = $BaseUrl.$FolderHrefs[$FolderIdx];
  my $NextPageHref;

	while ($StillPageToGo) {
		$StillPageToGo = 0;
		$Page++;

    Display("Handling page $Page.\n",MinVerbosity => 2);

    my ($PageAsString,$GetPageUrl) = GetPage(Url => $PageUrl);
    # To start with we limit us to a MessagesArea between
    # <table class="dItemListContentTable"..>  ... </table..>
    if ($PageAsString !~ m/<table class="dItemListContentTable"(.*?)<\/table/si) {
      die "Could not correctly parse the messages table.";
    }
    my $MessagesArea = $1;
    # In this message area there's the body of the table containing messages.
    # <tbody ..> ... </tbody..>
    if ($MessagesArea !~ m/<tbody\s*>(.*?)<\/tbody\s*>/si) {
      die "Could not correctly parse the messages table.";
    }
    $MessagesArea = $1;
    # MessagesArea now contains the body of the messages table.
    # Table rows <tr ..> .. : Description of the messages.
    # The class=".." tag hints on Unread or not.
    while ($MessagesArea =~ m/<tr(.*?)>/si) {
      $MessagesArea = $';
      if ($1 =~ m/ContentItemUnread/i) {
        $MessagesRead[$NrMessagesDetected] = 0;
      } else {
        $MessagesRead[$NrMessagesDetected] = 1;
      }
      # Goto 5th column.(to get the from)
      my $TdLine = "";
      for (my $Idx=0;$Idx<4;$Idx++) {
        $MessagesArea =~ m/<td(.*?)>(.*?)<\/td\s*>/i;
        $MessagesArea = $';
        $TdLine = $2;
      }
      if ($TdLine !~ m/class=\"from\"><span>(.*?)<\/span>/si) {
        die "Parse error for 'from'.";
      } 
      my $From = HtmlUnescape($1);
      Display("From '$From'.\n",MinVerbosity => 10);
      $MessagesFrom[$NrMessagesDetected] = $From;

      # Further to the subject column. There we pick up also the href of the message.
      $MessagesArea =~ m/<td(.*?)>(.*?)<\/td\s*>/i;
      $MessagesArea = $';
      $TdLine = $2;
      if ($TdLine !~ m/<a href=\"(.*?)\"\s*>(.*?)<\/a>/si) {
        die "Parse error for 'subject'.";
      }
      my $Subject = HtmlUnescape($2);
      my $Href    = $1;
      Display("Subject '$Subject'.\n",MinVerbosity => 10);
      Display("Href '$Href'.\n",MinVerbosity => 10);
      Display("Read '$MessagesRead[$NrMessagesDetected]'.\n",MinVerbosity => 10);
      $MessagesSubject[$NrMessagesDetected] = $Subject;
      if ($Href !~ m/ReadMessageId=(.*?)&/si) {
        if ($Href =~ m/EditMessageLight.aspx/si) {
          # We are probably in the 'concepts folder. 
          # Cannot be read ! We are simply done ...
          # But sometimes a EditMessage is stored in a folder .. not to loose the remainder
          # we look further. Therefore next, and no return.
          $NrEditsDetected++;
          next;
        }
        die "Could not parse message id.";
      } 
      my $MessageId = $1;
      Display("MessageId '$MessageId'.\n",MinVerbosity => 10);
      $MessagesId[$NrMessagesDetected] = $MessageId;

      $NrMessagesDetected++;
    }

    Display("Total messages reported : $ReportedNrMessages.\n" .
            "Nr messages detected    : $NrMessagesDetected.\n" .
            "Nr edits detected       : $NrEditsDetected.\n",
            MinVerbosity => 10);

    # If the number of messages we detected already is smaller than the
    # reported total , we still have to look for another page and reloop.
    if (($NrMessagesDetected + $NrEditsDetected) < $ReportedNrMessages) {
      $StillPageToGo = 1;
      Display("Search for one more page.\n",MinVerbosity => 10);
      # Search for 'next page' href in something like  (we orient on 'NextPageGlyph')
      # <a href="Foo"><img class="NextPageGlyph" src="FooBar" alt="Volgende pagina"title="Volgende pagina"/></a>
      if ($PageAsString !~ m/<a href=\"([^\"]+)\"><img class=\"NextPageGlyph\" .*?><\/a>/si) {
        die "Could not find an expected next page href.";
      }
      $NextPageHref = $1;
      Display("Html construct with 'Next Page' in it : '$&'.\n",MinVerbosity => 100);
      Display("Next page href : '$NextPageHref'.\n",MinVerbosity => 10);
      
      # Let's loop with the URL of the next page.
      $PageUrl = $BaseUrl . $NextPageHref;
    }
	}
}

########################################################################################################################
# 
# Get the messages from the folder with Idx as argument.
# Dead variant.
# 
########################################################################################################################

sub GetMessagesFromFolderDead($) {
  my ($FolderIdx)        = @_;
  my $FolderName         = $FolderNames[$FolderIdx];
  my $ReportedNrMessages = $FolderNrMessages[$FolderIdx];

	Display("Loading folder '$FolderName'.\n",MinVerbosity => 1);

	my $Page = 0;
	my $StillPageToGo = 1;

  # Reinitialize the global variable back to 0.
  $NrMessagesDetected = 0;
  $NrEditsDetected    = 0;

  my $PageUrl = $BaseUrl.$FolderHrefs[$FolderIdx]."\&page=1";

	while ($StillPageToGo) {
		$StillPageToGo = 0;
		$Page++;

    Display("Handling page $Page.\n",MinVerbosity => 2);

    my ($PageAsString,$GetPageUrl) = GetPage(Url => $PageUrl);
    # To start with we limit us to a MessagesArea between
    # <table ... id=MsgTable>  ... </table..>
    if ($PageAsString !~ m/<table.*?id=MsgTable>(.*?)<\/table/si) {
      die "Could not correctly parse the messages table.";
    }
    my $MessagesArea = $1;
    # MessagesArea now contains the body of the messages table.
    # Table rows <tr (bgcolor=..) name="Sender@Domain"..> .. : Description of the messages.
    while ($MessagesArea =~ m/<tr\s+(bgcolor=[^ ]+ )?\s*name=\"(.*?)\".*?>/si) {
      $MessagesArea = $';
      my $From = HtmlUnescape($2);
      $MessagesFrom[$NrMessagesDetected] = $From;
      Display("From '$From'.\n",MinVerbosity => 10);
      # Goto 7th column.(to get the href and from)
      my $TdLine = "";
      for (my $Idx=0;$Idx<6;$Idx++) {
        $MessagesArea =~ m/<td(.*?)>(.*?)<\/td\s*>/i;
        $MessagesArea = $';
        $TdLine = $2;
      }
      if ($TdLine !~ m/<a href=\"javascript:G\('\/cgi-bin\/(getmsg.*?)'\)\">.*?<\/a>/si) {
        if ($TdLine =~ m/<a href=\"javascript:G\('\/cgi-bin\/(compose.*?)'\)\">.*?<\/a>/si) {
          # We are probably in the 'concepts folder. 
          # Cannot be read ! We are simply done ...
          # But sometimes a EditMessage is stored in a folder .. not to loose the remainder
          # we look further. Therefore next, and no return.
          $NrEditsDetected++;
          next;
        }
        die "Parse error for 'href'.";
      }
      my $Href = $1;
      $MessagesHref[$NrMessagesDetected] = $Href;
      if ($Href !~ m/msg=(.*?)(&|$)/si) {
        die "Could not parse message id.";
      } 
      my $MessageId = $1;
      Display("MessageId '$MessageId'.\n",MinVerbosity => 10);
      $MessagesId[$NrMessagesDetected] = $MessageId;

      if ($Href =~ m/msgread/i) {
        $MessagesRead[$NrMessagesDetected] = 1;
      } else {
        $MessagesRead[$NrMessagesDetected] = 0;
      }

      # Further to the subject column.
      if ($MessagesArea !~ m/<td.*?>(.*?)<\/td\s*>/i) {
        die "Could not parse subject column.";
      }
      $MessagesArea = $';
      my $Subject = HtmlUnescape($1);
      Display("Subject '$Subject'.\n",MinVerbosity => 10);
      Display("Read '$MessagesRead[$NrMessagesDetected]'.\n",MinVerbosity => 10);
      $MessagesSubject[$NrMessagesDetected] = $Subject;

      $NrMessagesDetected++;
    }

    Display("Total messages reported : $ReportedNrMessages.\n" .
            "Nr messages detected    : $NrMessagesDetected.\n" .
            "Nr edits detected       : $NrEditsDetected.\n",
            MinVerbosity => 10);

    # If the number of messages we detected already is smaller than the
    # reported total , we still have to look for another page and reloop.
    if (($NrMessagesDetected + $NrEditsDetected) < $ReportedNrMessages) {
      $StillPageToGo = 1;
      Display("Search for one more page.\n",MinVerbosity => 10);
      # We search the next page url via the nextpg.gif and the javascript tag.
      if ($PageAsString !~ 
          m/<a href=\"javascript:\w*\('(page=[^']*)'\)"([^>]*)><img src=\"http:\/\/([^\/]*)\/i\.p\.nextpg\.gif\"/si){
        Display("PageAsString : '$PageAsString'.\n",MinVerbosity => 10);
        die "Could not match 'Next Page' string.";
      }
      # Calculation of the new page url.
      Display("Got '$1'.\nOut of '$&'.\n",MinVerbosity => 10);
      $PageUrl = $BaseUrl.$FolderHrefs[$FolderIdx]."\&".$1;
    }
	}
}

########################################################################################################################
# 
# Process the messages retrieved from a folder.
# Acts on global variables @Messages ...
# It just takes FolderIdx for knowing the name. (and now also for the MoveToFolder command)
# 
########################################################################################################################

sub ProcessMessagesFromFolder ($) {
  my ($FolderIdx) = @_;
  my $FolderName = $FolderNames[$FolderIdx];
  # Now let's run through all detected messages ..
  my $MessageIdx;
  for ($MessageIdx=0;$MessageIdx<$NrMessagesDetected;$MessageIdx++) {

    if ($DownloadedIdsFile) {
      # First we check and or create the file with the downloaded Ids.
      if (not -e $DownloadedIdsFile) {
        open (DOWNLOADED,">$DownloadedIdsFile") || die "Could not open $DownloadedIdsFile : $!.";
        print DOWNLOADED "-- This is an automatically generated file by $0 containing the id of downloaded messages\n";
        close (DOWNLOADED);
      }
     
      # Run through the downloaded Ids to check if we still have to download.
      my $HaveMessageAlready = 0;  
      open (DOWNLOADED,"$DownloadedIdsFile") || die "Could not open $DownloadedIdsFile : $!.";
      while(my $TmpId = <DOWNLOADED>) {
        chomp ($TmpId);
        if (uc($MessagesId[$MessageIdx]) eq uc($TmpId)) {
          $HaveMessageAlready = 1;
          Display("The message $MessageIdx with id '$TmpId' is already downloaded.\n",MinVerbosity => 10);
          last;
        }
      }
      close (DOWNLOADED);
  
      # All with this message if we downloaded already.
      next if ($HaveMessageAlready);
    }

    next if ( ($FetchOnlyUnread =~ m/Yes/i) && ($MessagesRead[$MessageIdx] == 1) );

    # Identifying a bit the message for the log.
    Display("Handling mail\n".
            "  from    : '$MessagesFrom[$MessageIdx]'\n".
            "  subject : '$MessagesSubject[$MessageIdx]'\n",MinVerbosity => 1);

    # JDLA getEmail , provided that HaveMessageAlready was not set.
    my $Message = GetEmail($MessageIdx,$FolderName);

    # Pipe it through a processor such as procmail.
    Display("Sending mail to '$MailProcessor'.\n",MinVerbosity => 1);
		open PR,"|$MailProcessor";
		print PR $Message;
		close PR || die "Sending mail to '$MailProcessor' did not succeed. See error log.";

    if ($DownloadedIdsFile) {
      # We don't have it yet. Add it to the downloaded.
      open (DOWNLOADED,">>$DownloadedIdsFile") || die "Could not open $DownloadedIdsFile : $!.";
      print DOWNLOADED "$MessagesId[$MessageIdx]\n";
      close (DOWNLOADED);
    }

    # And maybe we have to mark it read too ?
    if ($MarkRead =~ m/^Yes$/i) {
      MarkRead($MessageIdx);
    }
 
    # Maybe we even have to move it !
    if ($MoveToFolder ne "") {

      # If MoveToFolder is of the format @FileName, get the folder name from that FileName.
      if ($MoveToFolder =~ m/^@(.*)$/) {
        my $MoveToFolderName = $1;
        open(IN,$MoveToFolderName) || die "Could not open '$MoveToFolderName' : $!";
        $MoveToFolder = <IN>;
        chomp $MoveToFolder;
        close(IN);
      }

      # Do the move.
      MoveToFolder($MessageIdx,$MoveToFolder,$FolderIdx);
    }

		Display("Done.\n",MinVerbosity => 1);
	}
}

########################################################################################################################
# 
# Get a list of the folders we have to deal with and parse them one by one.
# Switcher between Dead and Live variant.
# 
########################################################################################################################

sub GetFolders() {
  if ($Mode =~ /Dead/i) {
    GetFoldersDead();
  } elsif ($Mode =~ /Live/i) {
    GetFoldersLive();
  } else {
    die "Not supported mode '$Mode'.";
  }
}

########################################################################################################################
# 
# Get a list of the folders we have to deal with and parse them one by one.
# Hotmail Live variant.
# 
########################################################################################################################

sub GetFoldersLive() {
  my ($FolderPageAsString,$GetPageUrl) = GetPage(Url => "${BaseUrl}ManageFoldersLight.aspx?",FollowForward => 1);
  # Scan the line for all folders, their href and title.
  # NrFolders on the fly;
  while ($FolderPageAsString =~ 
         m/<td class=\"dManageFoldersFolderNameCol\"><a\s*href=\"([^\"]*)\"\s*>(.*?)<\/a>\s*<\/td>\s*<td class=\"dManageFoldersTotalCountCol\">(\d+)<\/td>/gc) { 
    $FolderHrefs[$NrFolders]      = $1;
    $FolderNames[$NrFolders]      = $2;
    $FolderNrMessages[$NrFolders] = $3;
    Display(
     "Folder $NrFolders - $FolderHrefs[$NrFolders] - $FolderNames[$NrFolders] - $FolderNrMessages[$NrFolders].\n", 
      MinVerbosity => 10);
    $NrFolders++;
  }
}

########################################################################################################################
# 
# Get a list of the folders we have to deal with and parse them one by one.
# Hotmail Dead variant.
# 
########################################################################################################################

sub GetFoldersDead() {
  my ($FolderPageAsString,$GetPageUrl) = GetPage(Url => "$BaseUrl/folders?",FollowForward => 1);
  # Scan the line for all folders, their href and title.
  # NrFolders on the fly.
  while ($FolderPageAsString =~ 
         m/<td[^>]*><a\s*href=\"\/cgi-bin\/(HoTMaiL\?.*?)\"\s*>(.*?)<\/a>.*?<\/td>\s*<td[^>]*>(\d+)<\/td>/gc) { 
    $FolderHrefs[$NrFolders]      = $1;
    $FolderNames[$NrFolders]      = $2;
    $FolderNrMessages[$NrFolders] = $3;
    Display(
     "Folder $NrFolders - $FolderHrefs[$NrFolders] - $FolderNames[$NrFolders] - $FolderNrMessages[$NrFolders].\n", 
      MinVerbosity => 10);
    $NrFolders++;
  }
}
  
########################################################################################################################
# 
# The 'main' program.
#
########################################################################################################################

# Don't allow others to read our temp files
umask(077);
# The temporary directory creation.
mkdir($TmpDir) || die "Could not create $TmpDir : $!.";

ParseArgs();
ParseConfig();
DisplayIntroText();
Login();
GetFolders();
for (my $FolderIdx=0;$FolderIdx<$NrFolders;$FolderIdx++) {
  next if (scalar keys %FoldersToProcess && not exists $FoldersToProcess{lc $FolderNames[$FolderIdx]});
  Display("\nProcessing folder $FolderNames[$FolderIdx].\n",MinVerbosity => 1);
  GetMessagesFromFolder($FolderIdx);
  ProcessMessagesFromFolder($FolderIdx);  # Takes no arguments, works on globals. FolderIdx just for name calculation.
}
Display("All done.\n",MinVerbosity => 1);
CleanTempFiles();

exit(0);

########################################################################################################################
# 
# $Log: GetLive.pl,v $
# Revision 1.23  2007/06/24 17:37:41  jdla
#
# *) Bug 1742447 : Could not find expected url.
#    (After change of interface by MS)
# *) Bug 1742493 : GetLive doesn't die on wrong 'MailProcessor'.
#
# Revision 1.22  2007/06/19 20:30:26  jdla
# *) Bug 1739263 : --verbosity 0 should be silent.
# *) Request 1724728 : only fetch unread messages w/o id file
#
# Revision 1.21  2007/05/24 19:13:03  jdla
#
# *) Bug 1722346 : MoveToFolder : sometimes read , sometimes not read.
#
# Revision 1.20  2007/05/23 22:02:18  jdla
# *) Bug 1722346 : MoveToFolder : sometimes read, sometimes not read.
#    Now for sure (and thanks to a tool Live Http Headers in Mozilla)
#    solved decently for the Live branch.
#
# Revision 1.19  2007/05/22 19:49:07  jdla
# *) Bug 1722346 : MoveToFolder : sometimes read , sometimes not read.
#    Solved (I think ...) for the 'Dead' (old gotmail) mode.
#
# Revision 1.18  2007/05/20 18:53:47  jdla
# *) MoveToFolder now possible on downloading.
#
# Revision 1.17  2007/05/20 12:45:34  jdla
# Merged in the 1.16.2.1 that was by mistake done on a the release branch.
#
# Revision 1.16.2.1  2007/05/20 12:39:34  jdla
# *) MarkRead is now possible on downloading.
#
# Revision 1.16  2007/05/18 17:22:43  jdla
# *) Request 1721287 : Folder selection
#
# Revision 1.15  2007/05/18 14:59:21  jdla
# *) Bug 1719819 : Improve error message if Downloaded not specified.
#
# Revision 1.14  2007/05/18 14:10:40  jdla
# *) After the problem of Alex [dahaas] in which gotmail (the predecessor
#    of GetLive) was not able to correctly load his account, an overhaul
#    was made for correcting the counting of the messages per folder and
#    for detection of the correct NextPage url (page=n&wo=...) in his case.
#    Confirmed working for him and no regression for me.
#
# Revision 1.13.2.7  2007/05/18 12:19:12  jdla
# Overhaul in the detection of the number of messages per folder.
# Now as per suggesion of Alex [Dahaas] (dahaas@hotmail.com) based
# on the 'Manage Folders' function that is in Hotmail (the old and
# the live one).
#
# Revision 1.13.2.6  2007/05/18 08:10:43  jdla
# Further moving around of debug output for Alex' problem.
#
# Revision 1.13.2.5  2007/05/18 07:37:18  jdla
# Additional debug output ...
#
# Revision 1.13.2.4  2007/05/16 15:04:22  jdla
# Removed wrong debugoutput in a non-matched branch of if statement.
#
# Revision 1.13.2.3  2007/05/16 14:06:45  jdla
# Some additional debug info and some further adaptation of the pattern
# matching for NrMessages detection.
#
# Revision 1.13.2.2  2007/05/16 13:12:57  jdla
# Some improved debug info.
# NextPage search : corrected non-greedy search to negated class search.
#
# Revision 1.13.2.1  2007/05/15 21:03:56  jdla
# Try solving a stubborn problem coming from gotmail :
#   [ 1714743 ] Gotmail fails to download if box contains >100 messages
#
# Revision 1.13  2007/05/14 17:29:31  jdla
# *) Support 1717590 : error message => Classic named Dead now.
#
# Revision 1.12  2007/05/12 09:47:55  jdla
# *) Support 1717590 : error message => Improved error message.
#
# Revision 1.11  2007/05/07 18:27:52  jdla
# *) Bug 1714417 : execution fails if the config file name contains a dot
#
# Revision 1.10  2007/05/05 11:48:42  jdla
#
# *) Bug 1713304 : Strange characters in 'Processing folder Verwijderd'.
#
# Revision 1.9  2007/05/04 19:00:57  jdla
#
# *) Bug 1712959 : GetLive chokes on hotmail folders with 'Concepts' in it.
# *) Bug 1712958 : File with Ids incompatible between gotmail and GetLive.
#
# Revision 1.8  2007/05/02 22:01:23  jdla
# Comparison of downloaded Ids on a case independent way.
#
# Revision 1.7  2007/04/22 15:32:10  jdla
# Changed some MinVerbosity settings.
# Spelling error corrected.
# Reported Messages calculation only on first page !
#
# Revision 1.6  2007/04/22 10:17:19  jdla
# Corrected problem with fetching unread mail on Classic.
# (due to line colour attribute).
# Corrected problem of one page mailboxes by assuming that the
# number of reported messages equals the number of found messages.
# (but still warning on the situation to crosscheck)
#
# Revision 1.5  2007/04/20 22:24:28  jdla
# Added Name keyword for release tracking.
#
# Revision 1.4  2007/04/19 20:40:30  jdla
# Correction of the DisplayIntroText.
#
# Revision 1.3  2007/04/19 19:04:04  jdla
# Added case insensitivity to Mode check.
# Removed a MainPageAsString redefinition that screwed up Live mailbox
# fetch.
# Added initialization of NrMessagesDetected to the
# GetMessagesFromFolderLive.
#
# Revision 1.2  2007/04/18 21:49:22  jdla
# Bug correction : Initialization of NrMessagesDetected before each
# folder.
#
# Revision 1.1.1.1  2007/04/18 18:58:10  jdla
# Initial version of GetLive
#
# 
########################################################################################################################

# vim:et:sw=2:ts=2:filetype=perl:columns=120:lines=50:
